// (C) 2003 Ulrich Hecht <uli@emulinks.de>
// Licensed under the terms of the GNU Public License version 2

#include <stdio.h>
#include <assert.h>
#include <string.h>

#include "f2a.h"
#include <unistd.h>
#include <stdlib.h>

#define MODE_WRITE 0
#define MODE_READ 1

int verbose=1;

int main(int argc, char** argv)
{
	recvmsg rm;
	int opt;
	int mode=MODE_WRITE;	// default is to write to cart
	int sram=0;				// default is to access ROM, not SRAM
	int size=8;				// 8 MBit/kBit
	int offset=0;			// offset in MBit (ROM) or kBit (SRAM)

	assert(sizeof(sendmsg)==63);
	assert(sizeof(recvmsg)==64);

	do
	{
		opt=getopt(argc, argv, "rwbs:o:v");
		switch(opt)
		{
		case 'r':
			mode=MODE_READ;
			break;
		case 'w':
			mode=MODE_WRITE;
			break;
		case 'b':
			sram=1;
			break;
		case 's':
			size=atoi(optarg);
			break;
		case 'o':
			offset=atoi(optarg);
			break;
		case 'v':
			verbose++;
			break;
		case '?':
			fprintf(stderr, "f2a " F2A_VERSION "\n");
			fprintf(stderr, "Usage: %s [OPTION]... [FILE]...\n\n",argv[0]);
			fprintf(stderr, "  -r\tread from cart\n");
			fprintf(stderr, "  -w\twrite to cart (default)\n");
			fprintf(stderr, "  -b\taccess SRAM (default is to access ROM)\n");
			fprintf(stderr, "  -s\tamount of data to read (MBit for ROM, kBit for SRAM)\n");
			fprintf(stderr, "  -o\toffset on cart (MBit for ROM, kBit for SRAM)\n");
			fprintf(stderr, "  -v\tincrease verbosity\n");
			exit(1);
		}
	} while(opt!=-1);

	//printf("DEBUG: argc %d optind %d\n", argc, optind);

	if(argc-optind<=0)
	{
		fprintf(stderr, "%s: file name missing\n",argv[0]);
		exit(2);
	}
	if(mode==MODE_READ && argc-optind != 1)
	{
		fprintf(stderr, "%s: cannot write to multiple files\n",argv[0]);
		exit(3);
	}

	memset(&rm,0,sizeof(rm));

	if(usb_connect())
	{
		fprintf(stderr, "%s: unable to connect to F2A linker\n",argv[0]);
		exit(1);
	}

	f2a_info(&rm);

	if(rm.data[0]==0)
	{
		if(verbose) fprintf(stderr, "Please turn on GBA with SELECT and START held down.\n");
		f2a_boot(FIRMWARE_GBA);
		f2a_info(&rm);
	}


	if(mode==MODE_WRITE)
	{
		if(sram==0)
			f2a_writemem(argc-optind,argv+optind,0x8000000+offset*128*1024);
		else
			f2a_writemem(argc-optind, argv+optind, 0xe000000+offset*128);
	}
	else
	{
		if(sram==0)
			f2a_readmem(argv[optind], 0x8000000+offset*128*1024, size*128*1024);
		else
			f2a_readmem(argv[optind], 0xe000000+offset*128, size*128);
	}

	usb_disconnect();

	return 0;
}
