// OS X port (C) 2003 Eli Curtz <eli@nuprometheus.com>
// Original Linux version (C) 2003 Ulrich Hecht <uli@emulinks.de>
// Licensed under the terms of the GNU Public License version 2

IOReturn usb_connect();
void usb_disconnect();

IOReturn usb_read(char *buffer, int buffer_size);
IOReturn usb_write(char *buffer, int buffer_size);

#define GBA_EWRAM 0x02000000
#define GBA_IWRAM 0x03000000
#define GBA_VRAM 0x06000000
#define GBA_ROM 0x08000000
#define GBA_SRAM 0x0E000000

typedef struct
{
	unsigned int command;			// command to execute, see below
	unsigned int size;			// size of data block to read/write
	unsigned int _reserved1[2];
	unsigned int magic;			// magic number, see below
	unsigned int _reserved2[3];
	unsigned int unknown;			// no idea what this is for, seems to have to be 0xA for write, 0x7 for read
	unsigned int address;			// base address for read/write
	unsigned int sizekb;			// size of data block to read/write in kB
//unsigned char _reserved3[5*4];
	unsigned char _reserved3[5*4-1];	// for some reason the original software uses a 63 bytes structure for outgoing messages,
                                                // not 64 as it does for incoming messages, hence the "-1". It all seems to work fine with
                                                // 64 bytes, too, and I therefore suspect this to be a bug in the original SW.
} __attribute__ ((packed)) sendmsg;

typedef struct
{
	unsigned char data[64];
} recvmsg;

#define CMD_GETINF 5			// get info on the system status
#define CMD_WRITEDATA 6			// write data to RAM/ROM/SRAM
#define CMD_READDATA 7			// read data from RAM/ROM/SRAM
#define CMD_MULTIBOOT1 0xFF		// boot up the GBA stage 1, no parameters
#define CMD_MULTIBOOT2 0		// boot up the GBA stage 2, sendmsg->size has to be set

#define MAGIC_NUMBER 0xA46E5B91		// needs to be properly set for almost all commands

IOReturn f2a_sendMessage(sendmsg* msg);
IOReturn f2a_receiveMessage(recvmsg* msg);

IOReturn f2a_boot(FILE* bootfile);
IOReturn f2a_info(recvmsg* rm);
IOReturn f2a_writeMultiboot(FILE* file);
IOReturn f2a_writeRoms(UInt8 numFiles, FILE** files, UInt32 address);
IOReturn f2a_readMemory(FILE* file, UInt32 address, UInt32 size);

